<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2009, Skalfa LLC
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Skalfa LLC nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * 
 * @author Sergey Kambalin <greyexpert@gmail.com>
 * @package ow_plugins.newsfeed.bol
 * @since 1.0
 */
class NEWSFEED_BOL_Service
{
    const PRIVACY_SITE = 1;
    const PRIVACY_FOLLOW = 2;
    const PRIVACY_AUTOR = 4;
    const PRIVACY_FEED = 8; 
    
    const PRIVACY_FULL = 15;
    
    const ACTION_STATUS_ACTIVE = 'active';
    const ACTION_STATUS_INACTIVE = 'inactive';
    
    private static $classInstance;

    /**
     * Returns class instance
     *
     * @return NEWSFEED_BOL_Service
     */
    public static function getInstance()
    {
        if ( null === self::$classInstance )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }
    
    /**
     * 
     * @var NEWSFEED_BOL_ActionDao
     */
    private $actionDao;
    
    /**
     * 
     * @var NEWSFEED_BOL_FollowDao
     */
    private $followDao;
    
    /**
     * 
     * @var NEWSFEED_BOL_ActionFeedDao
     */
    private $actionFeedDao;
    
    /**
     * 
     * @var NEWSFEED_BOL_LikeDao
     */
    private $likeDao;
    
    /**
     * 
     * @var NEWSFEED_BOL_StatusDao
     */
    private $statusDao;
    
    private function __construct()
    {
        $this->actionDao = NEWSFEED_BOL_ActionDao::getInstance();
        $this->actionFeedDao = NEWSFEED_BOL_ActionFeedDao::getInstance();
        $this->followDao = NEWSFEED_BOL_FollowDao::getInstance();
        $this->likeDao = NEWSFEED_BOL_LikeDao::getInstance();
        $this->statusDao = NEWSFEED_BOL_StatusDao::getInstance();
    }

    public function saveAction( NEWSFEED_CLASS_Action $action )
    {
        $dto = $action->saveToDto();
        $this->actionDao->save($dto);
        
        $action->loadFromDto($dto);
        
        return $action;
    }
    
    /**
     * 
     * @param int $actionId
     * @param string $feedType
     * @param int $feedId
     * @return NEWSFEED_BOL_ActionFeed
     */
    public function addActionToFeed( NEWSFEED_CLASS_Action $action, $feedType, $feedId )
    {
        $actionFeed = new NEWSFEED_BOL_ActionFeed();
        $actionFeed->actionId = (int) $action->getId();
        $actionFeed->feedType = trim($feedType);
        $actionFeed->feedId = (int) $feedId;
        
        $this->actionFeedDao->addIfNotExists($actionFeed);
        
        return $actionFeed;
     }
    
    /**
     * 
     * @param string $entityType
     * @param int $entityId
     * @return NEWSFEED_CLASS_Action
     */
    public function getAction( $entityType, $entityId )
    {
        $dto = $this->actionDao->findAction($entityType, $entityId);
        
        return $this->convertDtoToAction($dto);
    }
    
    public function removeAction( $entityType, $entityId )
    {
        $dto = $this->actionDao->findAction($entityType, $entityId);
        
        if ( $dto === null )
        {
            return;
        }
        
        $this->actionFeedDao->deleteByActionId($dto->id);
        $this->likeDao->deleteByEntity($dto->entityType, $dto->entityId);
        $this->actionDao->delete($dto);
        
        $commentEntity = BOL_CommentService::getInstance()->findCommentEntity($dto->entityType, $dto->entityId);
        if ( $commentEntity->pluginKey == 'newsfeed' )
        {
            BOL_CommentService::getInstance()->deleteEntityComments($commentEntity->entityType, $commentEntity->entityId);
            BOL_CommentService::getInstance()->deleteCommentEntity($commentEntity->id);
        }
        
    }
    
    public function removeActionById( $id )
    {
        /* @var $dto NEWSFEED_BOL_Action */
        $dto = $this->actionDao->findById($id);
        
        $this->removeAction($dto->entityType, $dto->entityId);
    }
    
    public function removeActionListByPluginKey( $pluginKey )
    {
        $list = $this->actionDao->findByPluginKey($pluginKey);
        
        foreach ( $list as $dto )
        {
            /* @var $dto NEWSFEED_BOL_Action */
            $this->removeAction($dto->entityType, $dto->entityId);
        }
    }
    
    public function findExpiredActions( $inactivePeriod )
    {
        $this->actionDao->findExpired($inactivePeriod);
    }
    
    public function setActionStatusByPluginKey( $pluginKey, $status )
    {
        $this->actionDao->setStatusByPluginKey($pluginKey, $status);
    }
    
    /**
     * 
     * @param int $id
     * @return NEWSFEED_CLASS_Action
     */
    public function getActionById( $id )
    {
        $dto = $this->actionDao->findById($id);
        
        return $this->convertDtoToAction($dto);
    }
    
    public function getFeed( $feedType, $feedId, $count = null )
    {
        $list = $this->actionDao->findByFeed($feedType, $feedId, $count);
        
        return $this->convertDtoListToFeed($list);
    }
    
    public function findFeedCount( $feedType, $feedId )
    {
        return $this->actionDao->findCountByFeed($feedType, $feedId);
    }
    
    public function getUserFeed( $userId, $count = null )
    {
        $list = $this->actionDao->findByUser($userId, $count);
        
        return $this->convertDtoListToFeed($list);
    }
    
    public function findUserFeedCount( $userId )
    {
        return $this->actionDao->findCountByUser($userId);
    }
    
    public function getSiteFeed( $count = null )
    {
        $list = $this->actionDao->findSiteFeed($count);
        
        return $this->convertDtoListToFeed($list);
    }
    
    public function findSiteFeedCount()
    {
        return $this->actionDao->findSiteFeedCount();
    }
    
    public function isFollow( $userId, $feedType, $feedId )
    {
        return $this->followDao->findFollow($userId, $feedType, $feedId) !== null;
    }
    
    public function addFollow( $userId, $feedType, $feedId )
    {
        return $this->followDao->addFollow($userId, $feedType, $feedId);
    }
    
    public function removeFollow( $userId, $feedType, $feedId )
    {
        return $this->followDao->removeFollow($userId, $feedType, $feedId);
    }
    
    public function isLiked( $userId, $entityType, $entityId )
    {
        return $this->likeDao->findLike($userId, $entityType, $entityId) !== null;
    }
    
    public function findEntityLikesCount( $entityType, $entityId )
    {
        return $this->likeDao->findCountByEntity($entityType, $entityId);
    }
    
    public function findUserLikes( $userId )
    {
        return $this->likeDao->findByUserId($userId);
    }
    
    public function findEntityLikes( $entityType, $entityId )
    {
        return $this->likeDao->findByEntity($entityType, $entityId); 
    }
    
    public function findEntityLikeUserIds( $entityType, $entityId )
    {
        $likes = $this->findEntityLikes($entityType, $entityId);
        $out = array();
        
        foreach ( $likes as $like )
        {
            /* @var $like NEWSFEED_BOL_Like */
            $out[] = $like->userId;
        }
        
        return $out;
    }
    
    public function addLike( $userId, $entityType, $entityId )
    {
        return $this->likeDao->addLike($userId, $entityType, $entityId);
    }
    
    public function removeLike( $userId, $entityType, $entityId )
    {
        return $this->likeDao->removeLike($userId, $entityType, $entityId);
    }

    public function saveStatus( $feedType, $feedId, $status )
    {
        return $this->statusDao->saveStatus($feedType, $feedId, $status);
    }
    
    public function getStatus( $feedType, $feedId )
    {
        $dto = $this->findStatusDto( $feedType, $feedId );
        
        if ( $dto === null )
        {
            return null;    
        }
         
        return $dto->status;
    }
    
    /**
     * 
     * @param $feedType
     * @param $feedId
     * @return NEWSFEED_BOL_Status
     */
    public function findStatusDto( $feedType, $feedId )
    {
        return $this->statusDao->findStatus( $feedType, $feedId );
    }
    
    public function removeStatus( $feedType, $feedId )
    {
        $this->statusDao->removeStatus($feedType, $feedId);
    }
    
    private function convertDtoListToFeed( $actionDtoList )
    {
        $out = array();
        foreach ( $actionDtoList as $dto )
        {
            $out[$dto->id] = $this->convertDtoToAction($dto); 
        }
        
        return $out;
    }
    
    /**
     * 
     * @param NEWSFEED_BOL_Action $dto
     * @return NEWSFEED_CLASS_Action
     */
    private function convertDtoToAction( $dto )
    {
        if ( empty($dto) )
        {
            return null;
        }
        
        $action = new NEWSFEED_CLASS_Action();
        $action->loadFromDto($dto);
        
        return $action;
    }
}